/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * CLOCK.C
 *
 * This program demonstrates the use of the time and alarm functions
 * by implementing an alarm clock program on the device.
 *
 * This program also demonstrates:
 *    Turning the pager on/off
 *    Use of timers
 *    Dealing with foreground/background switches
 *    A good way of dealing with events in a very modal application
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include "Pager.h"
#include "popupmenu.h"  // Functions for sample module POPUPMENU.C
#include "clock.h"  // Bitmap definitions for clock digits

char VersionPtr[] = "Alarm Clock ";
int AppStackSize = 1500;

// Define an event code for the one second timer.
// Events above EVENT_USER are guaranteed not to conflict with 
// the system's internal event codes.

#define MY_TIMER_EVENT (EVENT_USER+1)

// Array of pointers to the digit bitmaps.

// Declaring the array as 'const' tells the compiler that the C code will never
// modify it.  As such, it ends up in FLASH, conserving precious RAM.
const BitMap *const digits[10] = {
  &Zero,
  &One,
  &Two,
  &Three,
  &Four,
  &Five,
  &Six,
  &Seven,
  &Eight,
  &Nine,
};

// Define how the program stores time for display and edit purposes:
#define TIME_LEN 25
static const char TimeFormat[TIME_LEN] = "24:59:59  Day 12-31-2030";

static BOOL IsForeground = FALSE;

static LcdConfig LcdConf;

//-----------------------------------------------------------------------------
// Display the time in big digits, with our without a cursor.
//-----------------------------------------------------------------------------
void DisplayTime(char *TimeString, int CursorPos)
{
  int i, x;

#define DIGITS_TOP 18

  for (i = 0, x = 0; i < 10; i++)
  {
    if (TimeString[i] == ':')
    {
      // Display large colon
      LcdCopyBitmapToDisplay(&Colon, x, DIGITS_TOP);
      x += 8;

    }
    else
    {

      if (TimeString[i] >= '0' && TimeString[i] <= '9')
      {
        // Display a large digit
        LcdCopyBitmapToDisplay(digits[TimeString[i] - '0'], x, DIGITS_TOP);
      }

      if (i == CursorPos)
      {
        // Draw a rectangle around the large digit.
        LcdDrawBox(DRAW_BLACK, x, DIGITS_TOP - 1, x + 15, DIGITS_TOP + 18);
      }

      x += 16;
    }
  }

  // Display the date in regular digits.

  if (CursorPos < 0)
  {
    LcdPutStringXY(10, DIGITS_TOP + 20, TimeString + 10, -1, 0);
  }
  else
  {
    // If the cursor is shown, don't display the week day, and
    // show an underscor cursor if the cursor is on the date portion.
    if (CursorPos >= 14)
    {
      int x;
      x = LcdGetStringWidth(TimeString + 14, -1, CursorPos - 14);

      LcdPutStringXY(x + 29, DIGITS_TOP + 22, "_", 1, 0);
    }
    LcdPutStringXY(30, DIGITS_TOP + 20, TimeString + 14, -1, 0);
  }
}

//-----------------------------------------------------------------------------
// Convert a time to a string.
//-----------------------------------------------------------------------------
void TimeToString(char *TimeString, TIME * Time)
{
  RimSprintf(TimeString, 10, "%2d:%02d:%02d  ", Time->hour, Time->minute, Time->second);
  RimSprintf(TimeString + 10, 15, "%s %02d-%02d-%4d", "Sun\0Mon\0Tue\0Wed\0Thu\0Fri\0Sat\0" + Time->day * 4, Time->month, Time->date, Time->year);
}

//-----------------------------------------------------------------------------
// Simple convert string to number (there is no scanf function)
//-----------------------------------------------------------------------------
int StringToNum(char *String)
{
  int res;

  res = 0;
  while (*String >= '0' && *String <= '9')
  {
    res = res * 10 + *(String++) - '0';
  }
 return res;
}

//-----------------------------------------------------------------------------
// Some messages are always processed the same way regardless of what mode
// the app is in.  This function handles messages.
//
// Note that RimRegisterMessageCallback can also be used for this purpose, but
// using it is slightly more complicated than this approach.
//-----------------------------------------------------------------------------
void MyGetMessage(MESSAGE * msg)
{
  for (;;)
  {
    RimGetMessage(msg);
    switch (msg->Event)
    {
      case RTC_ALARM_EXPIRED:
        // Sound the alarm!
        RimAlertNotify(TONE_SEQUENCE_3, 1);
        break;

      case SWITCH_FOREGROUND:
        // Start the seconds timer when foregrounded.
        RimSetTimer(MY_TIMER_EVENT, TICKS_PER_SECOND, TIMER_PERIODIC);
        IsForeground = TRUE;
        break;

      case SWITCH_BACKGROUND:
        // Kill the seconds timer when in background.  
        // This is to save battery power on the device.
        RimKillTimer(MY_TIMER_EVENT);
        IsForeground = FALSE;
        break;

      default:
        // Let mainline process the message.
        return;
    }
  }
}

//-----------------------------------------------------------------------------
// Display the main screen.
//-----------------------------------------------------------------------------
void DisplayCurrentTimeScreen(void)
{
  char TimeString[TIME_LEN];
  TIME CurrentTime;

  // Clear the display.
  LcdClearDisplay();
  LcdPutStringXY(0, 0, "Current time:", -1, 0);

  // Get the present time.
  RimGetDateTime(&CurrentTime);

  // Turn time into a string.
  TimeToString(TimeString, &CurrentTime);

  // Show the time in big digits.
  DisplayTime(TimeString, -1);


  // Display status of the alarm
  LcdPutStringXY(0, LcdConf.height - 10, "Alarm: ", -1, 0);

  // If an alarm is set, display when it will go off.
  // Note that RimGetAlarm gets the first alarm set for any task, so
  // if another task also has an alarm set, this example will display
  // that alarm time instead.

  // For normal applications, it would be better to just remember what the
  // alarm time was set to, but this app is more for demonstrating the use 
  // of API calls.

  if (RimGetAlarm(&CurrentTime) > 0)
  {
    TimeToString(TimeString, &CurrentTime);
    LcdPutStringXY(30, LcdConf.height - 10, TimeString, -1, 0);
  }
  else
  {
    LcdPutStringXY(30, LcdConf.height - 10, "Off", -1, 0);
  }
}

//-----------------------------------------------------------------------------
// Allow the user to enter the time.
//-----------------------------------------------------------------------------
int EnterTime(TIME * TimeToSet, char *ScreenTitle)
{
  char SetTimeString[TIME_LEN];
  int CursorPos;
  int a;
  BOOL ismax;
  MESSAGE msg;

  TimeToString(SetTimeString, TimeToSet);
  if (SetTimeString[0] == ' ')
    SetTimeString[0] = '0';

  CursorPos = 0;

  for (;;)
  {
  redraw:

    LcdClearDisplay();
    LcdPutStringXY(0, 0, ScreenTitle, -1, 0);

    // Check that all the numbers in the time string are
    // within range.

    ismax = TRUE;
    for (a = 0; a < TIME_LEN; a++)
    {
      if (TimeFormat[a] >= 0 && TimeFormat[a] <= '9')
      {
        if (ismax)
        {
          if (SetTimeString[a] < TimeFormat[a])
          {
            ismax = FALSE;
          }
          else
          {
            if (SetTimeString[a] > TimeFormat[a])
            {
              SetTimeString[a] = TimeFormat[a];
              // One of the numbers was out of range.
              RimSpeakerBeep(1000, 40, 50, 50);
            }
          }
        }
      }
      else
      {
        ismax = TRUE;
      }
    }

    DisplayTime(SetTimeString, CursorPos);

    MyGetMessage(&msg);
    switch (msg.Event)
    {
      case KEY_DOWN:
        if (msg.SubMsg == KEY_ENTER)
        {
          // A time was set.
          goto save_time;
        }

        // Turn upper row alpha characters into digits.
        // That way, the user doesn't have to push ALT.
        for (a = 0; a < 10; a++)
        {
          if (msg.SubMsg == (DWORD)"pqwertyuio"[a])
          {
            msg.SubMsg = a + '0';
          }
        }

        if (msg.SubMsg >= '0' && msg.SubMsg <= '9')
        {
          SetTimeString[CursorPos] = (char)msg.SubMsg;
        }
        else if (msg.SubMsg == KEY_BACKSPACE)
        {
          // Make backspace move the cursor left.
          msg.Event = THUMB_ROLL_DOWN;
        }
        else
        {
          RimSpeakerBeep(1000, 40, 50, 50);
          break;
        }

      // Fallthrough intentional to do a cursor move.

      case THUMB_ROLL_UP:
      case THUMB_ROLL_DOWN:

        // Keep scanning until the next valid deditable digit.
        for (;;)
        {
          if (msg.Event == THUMB_ROLL_DOWN)
          {
            CursorPos -= 1;
          }
          else
          {
            CursorPos += 1;
          }

          // Handle wrap arounds.
          if (CursorPos < 0)
            CursorPos = TIME_LEN - 1;
          if (CursorPos >= TIME_LEN)
            CursorPos = 0;

          // Check if cursor is on an editable digit.
          if (TimeFormat[CursorPos] <= '9' && TimeFormat[CursorPos] >= '0')
          {
            break;
          }
        }
        break;

      case THUMB_CLICK:
        // Note:
        // PupupMenuWaitForSelection can cause us to miss the selection.
        // However, alarms going off while being set don't make that
        // much sense, so its pretty safe.

        PopupMenuBegin("Hide menu|Save;Cancel", 1);
        for (;;)
        {
          int ret;
          MyGetMessage(&msg);

          ret = PopupMenuProcessMessage(&msg);
          switch (ret)
          {
            case 0:  // Hide menu
              goto redraw;

            case 1:  // Save time
              goto save_time;

            case 2:  // Cancel.
              return FALSE;
          }
        }
    }
  }

save_time:
  // Note:  This statement is reached via a goto.
  //        It is the author's opinion that using a flag to terminate a loop
  //        causes even more confusion than a goto does.

  // Convert it back to a time structure.
  TimeToSet->hour = (BYTE)StringToNum(SetTimeString + 0);
  TimeToSet->minute = (BYTE)StringToNum(SetTimeString + 3);
  TimeToSet->second = (BYTE)StringToNum(SetTimeString + 6);

  TimeToSet->month = (BYTE)StringToNum(SetTimeString + 14);
  TimeToSet->date = (BYTE)StringToNum(SetTimeString + 17);
  TimeToSet->year = (WORD)StringToNum(SetTimeString + 20);

  // A time was set.
  return TRUE;
}

//-----------------------------------------------------------------------------
// Handle turning pager off.
//-----------------------------------------------------------------------------
void DoShutdown(void)
{
  MESSAGE msg;
  // This function handles this app turning the pager off.

  // Note that under normal circumstances, in any set of apps, there should
  // only be one app that turns the pager off.  

  // This app does turning off to show that the alarm still goes off
  // When the pager is turned off.

  // If another app turns the pager off, this app doesn't call DoShutdown.
  // It will simply see and ignore the POWER_UP and POWER_DOWN messages, 
  // without giving the user feedback as to shutting down, as it would not
  // be the app that is causing the shutdown.

  LcdClearDisplay();
  LcdPutStringXY(LcdConf.width / 2, LcdConf.height / 2 - 4, "Shutting down...", -1, LCD_CENTERED);
  RimRequestPowerOff();

  for (;;)
  {
    MyGetMessage(&msg);
    switch (msg.Event)
    {
      case POWER_UP:
        // Unit was turned back on. Go back to regular processing.
        return;
    }
  }
}

//-----------------------------------------------------------------------------
// Handle popup menu in the main list.
//-----------------------------------------------------------------------------
void DoMainScreenMenu(void)
{
  MESSAGE msg;
  int ret;

  // PopupMenuBegin and PopupMenuProcessMessage are functions implemented
  // in POPUPMENU.C.  This is part of this sample app, not UI32.DLL

  PopupMenuBegin("Hide menu|Set time;Set Alarm;Alarm Off|Switch Apps|Shut down", 0);

  for (;;)
  {
    MyGetMessage(&msg);
    switch (msg.Event)
    {
      case MY_TIMER_EVENT:
        DisplayCurrentTimeScreen();

      // Fall through intentional.  PopupMenuProcessMessage
      // will redraw the menu on any event it doesn't care about,
      // so the menu gets re-displayed on top of the time.

      // Because LCD is double buffered, this doesn't cause flicker.

      default:

        ret = PopupMenuProcessMessage(&msg);
        if (ret != MENU_NO_SELECTION)
        {
          TIME SetTime;

          // A selection was made.
          switch (ret)
          {
            case 0:  // Hide menu.
              break;
            case 1:  // Set time
              RimGetDateTime(&SetTime);
              if (EnterTime(&SetTime, "Set time:"))
              {
                if (!RimSetTime(&SetTime))
                {
                  // Bad time.
                  RimSpeakerBeep(1000, 40, 50, 50);
                }
                if (!RimSetDate(&SetTime))
                {
                  // Bad year.
                  RimSpeakerBeep(1000, 40, 50, 50);
                }
              }
              break;
            case 2:  // Set alarm
              RimGetDateTime(&SetTime); // Initialize date & year of time.

              SetTime.second = 0; // Clear the second.  Second setting is ignored
            // by RimSetAlarm.

              SetTime.minute = (BYTE) ((SetTime.minute + 1) % 60);
            // Put time forward by one minute
            // if it's easy, so alarm will go off soon.


              RimGetAlarm(&SetTime);  // Get whatever the alarm was last set to.
            // If no alarm is set, this call does nothing.
            // Note that this function actually gets the 
            // first RTC alarm set by any task.

              if (EnterTime(&SetTime, "Set alarm"))
              {
                // Set the alarm.
                RimSetAlarmClock(&SetTime, TRUE);
              }
              break;

            case 3:  // Kill alarm.
              RimSetAlarmClock(&SetTime, FALSE);
              break;

            case 4:  // Turn off the pager.
              RimInvokeTaskSwitcher();
              break;

            case 5:  // Turn off the pager.
              DoShutdown();
              break;

          }
          // Redraw screen to get rid of menu.
          DisplayCurrentTimeScreen();
          return;
        }
    }
  }
}

//-----------------------------------------------------------------------------
// Main loop for this alarm clock application
//-----------------------------------------------------------------------------
void PagerMain(void)
{
  MESSAGE msg;

  LcdGetConfig(&LcdConf);

  // Turn on broadcast of clock messages for this task.
  // On the Inter@ctive pager 950 it is on by default, but future devices
  // will have it off by default.
  RimSetReceiveFromDevice(DEVICE_RTC, TRUE);

  // Give other applications an opportunity to grab the foreground.
  RimTaskYield();
  RimTaskYield();

  // If no app grabs the foreground, the system will simply pick one.
  // Check if this app is on the foreground now.
  if (RimGetCurrentTaskID() == RimGetForegroundApp())
  {
    // This app is in the foreground.
    RimSetTimer(MY_TIMER_EVENT, TICKS_PER_SECOND, TIMER_PERIODIC);
    IsForeground = TRUE;
    DisplayCurrentTimeScreen();
  }

  // Main loop of program.  Note that PagerMain runs for the existence of 
  // the pager, so it never returns.
  for (;;)
  {
    MyGetMessage(&msg);
    switch (msg.Event)
    {
      case RTC_CLOCK_UPDATE:
        if (IsForeground)
        {
          // Reset the periodic update to correspond to aligh it with
          // RTC clock second changes.  The RTC_CLOCK_UPDATE message is 
          // sent at the start of a minute, which is also the start of a 
          // second.

          // Setting a timer by the same timer ID as an existing one
          // kills the old timer.
          RimSetTimer(MY_TIMER_EVENT, TICKS_PER_SECOND, TIMER_PERIODIC);

          // Display the time (our time won't happen for another second)
          DisplayCurrentTimeScreen();
        }
        break;

      case MY_TIMER_EVENT:
        // Display current time.
        DisplayCurrentTimeScreen();
        break;

      case THUMB_CLICK:
        DoMainScreenMenu();
        break;
    }
  }
}
