/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * POPUPMENU.C
 *
 * Pop up a menu and allow the user to make a selection.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include <Pager.h>
#include "popupmenu.h"

static int NumItems;
static BYTE VOffsets[20];
static BYTE ItemLengths[20];
static BYTE SeparatorBars[20];
static char *MenuItems[20];
static int LeftEdge;
static int VScrollPix;
static int MenuHeight;
static int Selection;

static char UpArrow[] = "\x16";
static char DownArrow[] = "\x17";

static LcdConfig LcdConf;

//-----------------------------------------------------------------------------
// Draw the menu
//-----------------------------------------------------------------------------
void DrawMenu(void)
{
  int a;

  // Draw line to left of menu
  LcdDrawLine(DRAW_BLACK, LeftEdge - 1, 0, LeftEdge - 1, MenuHeight < LcdConf.height ? MenuHeight : LcdConf.height);

  if (MenuHeight < LcdConf.height)
  {
    // Draw line on bottom of menu.
    LcdDrawLine(DRAW_BLACK, LeftEdge, MenuHeight, LcdConf.width, MenuHeight);
  }

  // Clear out the menu area.
  LcdRasterOp(WHITE, LcdConf.width - LeftEdge, MenuHeight < LcdConf.height ? MenuHeight : LcdConf.height, NULL, 0, 0, NULL, LeftEdge, 0);

  // Range check the selection.
  if (Selection < 0)
    Selection = 0;
  if (Selection >= NumItems)
    Selection = NumItems - 1;

  if (VOffsets[Selection] + VScrollPix < 0)
  {
    // Scroll the menu up.
    VScrollPix = -VOffsets[Selection];
  }

  if (VOffsets[Selection] + VScrollPix + 9 > LcdConf.height)
  {
    // Scroll the menu down.
    VScrollPix = LcdConf.height - VOffsets[Selection] - 9;
  }

  // Display all of the menu items.
  for (a = 0; a < NumItems; a++)
  {
    int Vpos;
    Vpos = VOffsets[a] + VScrollPix;

    // Not visible - missing from top.
    if (Vpos < 0)
      continue;

    // Past the visible part.
    if (Vpos + 8 > LcdConf.height)
    {
      LcdPutStringXY(LcdConf.width, LcdConf.height - 9, DownArrow, 1, LCD_RIGHT_JUSTIFIED);
      break;
    }

    // Draw the separator bar.
    if (SeparatorBars[a])
    {
      if (Vpos + 11 < LcdConf.height)
      {
        LcdDrawLine(DRAW_BLACK, LeftEdge + 2, Vpos + 10, LcdConf.width, Vpos + 10);
      }
    }

    // Display the menu item.                                 
    LcdPutStringXY(LeftEdge + 2, Vpos, MenuItems[a], ItemLengths[a], 0);
  }

  if (VScrollPix)
  {
    // Show the up arrow if necessary
    LcdPutStringXY(LcdConf.width, 0, UpArrow, 1, LCD_RIGHT_JUSTIFIED);
  }

  // Highlight the current selection
  LcdRasterOp(INVERT_DEST, LcdConf.width - 1 - LeftEdge, 9, NULL, 0, 0, NULL, LeftEdge + 1, VOffsets[Selection] + VScrollPix);
}

//-----------------------------------------------------------------------------
// Start a popup menu,
//-----------------------------------------------------------------------------
void PopupMenuBegin(char *ItemString, int DefaultSelection)
{
  int a;

  LcdGetConfig(&LcdConf);

  NumItems = 0;
  VScrollPix = 0;
  LeftEdge = 0;

  // Parse the menu items out of the string.
  for (;;)
  {
    MenuItems[NumItems] = ItemString;
    VOffsets[NumItems] = (BYTE)VScrollPix;
    for (a = 0; a < 20; a++)
    {
      if (*ItemString == '|' || *ItemString == ';' || *ItemString == 0)
      {
        int temp;
        ItemLengths[NumItems] = (BYTE)a;
        // Find the widest element.
        temp = LcdGetStringWidth(MenuItems[NumItems], 0, a);
        if (temp > LeftEdge)
          LeftEdge = temp;

        VScrollPix += 9;
        SeparatorBars[NumItems] = 0;

        if (*ItemString == '|')
        {
          // There is a separator bar.
          VScrollPix += 3;
          SeparatorBars[NumItems] = 1;
        }

        if (++NumItems >= 20)
        {
          // Safety - don't allow more than 20 items.
          goto itemsdone;
        }

        if (*ItemString == 0)
          goto itemsdone;

        ItemString += 1;
        break;

      }
      ItemString += 1;
    }
  }

itemsdone:

  LeftEdge = LcdConf.width - LeftEdge - 2;

  VScrollPix = 0;

  MenuHeight = VOffsets[NumItems - 1] + 10;

  Selection = DefaultSelection;

  DrawMenu();
}

//-----------------------------------------------------------------------------
// Process menu messages
//-----------------------------------------------------------------------------
int PopupMenuProcessMessage(MESSAGE * msg)
{
  // Note: Since msg->Event already uniquely identifies the event,
  // no need to chech msg->Device field.

  switch (msg->Event)
  {

    case THUMB_ROLL_UP:
      Selection -= 1;
      break;

    case THUMB_ROLL_DOWN:
      Selection += 1;
      break;

    case THUMB_CLICK:
      return Selection;

    case KEY_DOWN:
      if (msg->SubMsg == KEY_BACKSPACE)
      {
        return MENU_CANCELED;
      }
      if (msg->SubMsg == KEY_ENTER)
      {
        return Selection;
      }
  }

  DrawMenu();

  return MENU_NO_SELECTION; // No selection was made.
}

//-----------------------------------------------------------------------------
// A message loop to handle a popup menu.  Note that this message loop 
// eats all other messages.  If an app needs to process other messages at
// the same time, it is best to call PopupMenuBegin and PupuMenuProcessMessage
// from your own message loop.
//-----------------------------------------------------------------------------
int PopupMenuWaitForSelection(char *Items, int DefaultItem)
{
  MESSAGE msg;
  int ret;

  PopupMenuBegin(Items, DefaultItem);

  for (;;)
  {
    RimGetMessage(&msg);

    ret = PopupMenuProcessMessage(&msg);
    // Any message that PopupMenuProcessMessage doesn't care about
    // is simply ignored.  If you need to process other messages,
    // implement your own PopupMenuWaitForSelection function.

    if (ret != MENU_NO_SELECTION)
    {
      // A selection was made.  Return with the selection.
      return ret;
    }
  }
}
