/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * PING.C
 *
 * This application runs with two threads for getting radio events to
 * demonstrate how the radio API behaves with respect to multiple threads
 * and multiple applications.
 * 
 * This app presents no user interface to speak of.  Instead, pushing various
 * keys exercises various function calls of the API.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/

#include "Pager.h"
#include "mobitex.h"
#include <string.h>

char VersionPtr[] = "Ping Test";
int  AppStackSize = 2000;

// Note: the following buffers are used by more than one threads.
// Because of the cooperative nature of the multitasker, this is possible
// so long as no thread assumes the data is valid across Yield or GetMessage
static BYTE mpakBuffer[560];
static MPAK_HEADER mpakHeader;

// Flag for getting out of a non get-message loop.  
static int BusyWaitFlag;

//------------------------------------------------------------------
// Put a line on the pager LCD and scroll.
//------------------------------------------------------------------
#define SCROLL_HEIGHT 8
void putline (char *str)
{
    LcdConfig lcdConf;
    LcdGetConfig (&lcdConf);

    LcdScroll(SCROLL_HEIGHT);

    // Display the next line.
    LcdPutStringXY( 0, lcdConf.height-SCROLL_HEIGHT, str, -1, TEXT_NORMAL );
}


//------------------------------------------------------------------
// Send a 'PING' MPAK to self
//------------------------------------------------------------------
int SendPing (void)
{
    MPAK_HEADER header;
    char buffer[30];
    RADIO_INFO info;
    int SentTag;

    RadioGetDetailedInfo(&info);

    memset((unsigned char *)&header, 0, sizeof(header));
    header.Destination = info.LocalMAN;
    header.MpakType = MPAK_TEXT;
    SentTag = RadioSendMpak(&header, (unsigned char *)"This is a test", 14);

    if (SentTag >= 0){
        RimSprintf(buffer, sizeof(buffer), "Submitted tag = %d", SentTag);
        putline(buffer);
    }else{
        putline("Submit failed");
    }
    return SentTag;
}

//------------------------------------------------------------------
// Get information from the radio
//------------------------------------------------------------------
void GetRadioInfo (void)
{
    char buffer[60];

    RADIO_INFO info;

    RadioGetDetailedInfo(&info);
    RimSprintf(buffer, sizeof(buffer), "MAN:%ld, Active=%d",
                                       info.LocalMAN, info.Active);
    putline(buffer);
    RimSprintf(buffer, sizeof(buffer), "GetRSSI = %d dBm", RadioGetSignalLevel());
    putline(buffer);
}


//------------------------------------------------------------------
// Message handling loop (main dispatcher)
//------------------------------------------------------------------
void RadioTestTask(void)
{
    char buffer[40];
    MESSAGE msg;
    int n;
    int lastSentTag;
    int StopOnNextRx;
    int retval;

    lastSentTag = -1;
    StopOnNextRx = FALSE;

    {
        static int Invocation = 0;
        PID pid;

        pid.Icon = NULL;

        if (Invocation == 0){
           pid.Name = "Ping 1";
           Invocation = 1;
        }else{
           pid.Name = "Ping 2";
        }

        pid.EnableForeground = 1;
        RimSetPID(&pid);
    }


    // announce our presence
    putline("Ping test running");
    putline("p=ping s=RSSI");

    // tell the OS that we want to receive radio events
    RadioRegister();

    // standard message loop structure for an application
    for (;;) {
        RimGetMessage( &msg );

        retval = 0;

        switch (msg.Device) {
            case DEVICE_KEYPAD:
                if (msg.Event == KEY_DOWN){
                    switch(msg.SubMsg){
                        case 'p':
                            // Send a PING.
                            retval = lastSentTag = SendPing();
                            break;

                        case 's':
                            // Get information from the radio
                            GetRadioInfo();
                            break;

                        case 'c':
                            // Cancel the most recently sent MPAK.
                            retval = RadioCancelSendMpak(lastSentTag);
                            putline("RadioCancelSendMpak()");
                            break;

                        case 'g':
                            // Use the RadioOnOff to get the current status.
                            // Note that RadioOnOff always returns the previous status, but if the
                            // status is not being set, this is the same as the current status.
                            putline("RadiOnOff(RADIO_GET_ONOFF)");
                            switch(retval = RadioOnOff(RADIO_GET_ONOFF)){
                                case RADIO_ON:
                                    putline("Radio is on");
                                    break;

                                case RADIO_OFF:
                                    putline("Radio is off");
                                    break;

                                case RADIO_LOWBATT:
                                    putline("Radio on, battery too low");
                                    break;

                            }
                            break;

                        case 'f':
                            // Turn the radio off
                            putline("RadiOnOff(RADIO_OFF)");
                            retval = RadioOnOff(RADIO_OFF);
                            break;

                        case 'o':
                            // Turn the radio on
                            putline("RadiOnOff(RADIO_ON)");
                            retval = RadioOnOff(RADIO_ON);
                            break;

                        case 'u':
                            // Unregister this task for radio events
                            putline("RadioDeregister()");
                            RadioDeregister();
                            break;

                        case 'r':
                            // Register this task for radio events
                            putline("RadioRegister()");
                            RadioRegister();
                            break;

                        case 'e':
                            // Ready to take MPAKs again
                            putline("RadioResumeReception()");
                            RadioResumeReception();
                            break;

                        case 't':
                            // Stop getting MPAKS from now on.
                            putline("RadioStopReception(-1)");
                            RadioStopReception(-1);
                            break;

                        case 'n':
                            // Next time we get an MPAK, 'unget' it and stop reception.
                            putline("Stop on next rx");
                            StopOnNextRx = TRUE;
                            break;

                        case 'b':
                            // Go into a busywait, so that no MPAK messages
                            // are received, to simulate getting behind.
                            if (BusyWaitFlag){
                                // Other thread already busywaiting.  Cancel busywait,
                                // or it is not possible to leave this state anymore.
                                BusyWaitFlag = FALSE;
                                putline("unbusy other task");
                            }else{
                                // Go into busywait if other thread is not.
                                putline("busywait started");
                                BusyWaitFlag = TRUE;
                                while(BusyWaitFlag){
                                    RimTaskYield();
                                }
                                putline("busywait done");
                            }
                            break;

                        case '0':
                        case '1':
                        case '2':
                        case '4':
                        case '8':
                            // Set skipnum commands.  0 is request only.
                            {
                                int LastSkipnum;
                                char buffer[50];
                                retval = LastSkipnum = RadioRequestSkipnum(NULL, msg.SubMsg-'0');

                                RimSprintf(buffer,sizeof(buffer),"Skipnun set %d was %d",
                                                            msg.SubMsg-'0',LastSkipnum);

                                putline(buffer);
                            }
                            break;

                        case KEY_BACKSPACE:
                            // Allow user to switch to the other thread.
                            RimInvokeTaskSwitcher();
                            break;

                        default:
                            putline("Unknown keystroke");

                    }
                }
                break;

            case DEVICE_RADIO:
                switch (msg.Event) {
                    case MESSAGE_RECEIVED:
                        // mpakTag = msg.SubMsg
                        // type = msg.Data[0]
                        // HPID = msg.Data[1]
                        RimSprintf(buffer, sizeof(buffer),
                                        "Received %d/%d/%d",
                                        msg.SubMsg, msg.Data[0], msg.Data[1]);
                        // putline(buffer);

                        // show packet timestamp
                        RimSprintf(buffer, sizeof(buffer),
                                        "TS:%d/%02d/%02d %02d:%02d:%02d",
                                        mpakHeader.Timestamp.year,
                                        mpakHeader.Timestamp.month,
                                        mpakHeader.Timestamp.date,
                                        mpakHeader.Timestamp.hour,
                                        mpakHeader.Timestamp.minute,
                                        mpakHeader.Timestamp.second);
                        // putline(buffer);

                        // retrieve the packet
                        n = RadioGetMpak(msg.SubMsg, &mpakHeader, mpakBuffer);
                        if (n < 0) {
                            putline("RadioGetMpak failed");
                            break;
                        }

                        if (memcmp((unsigned char *)mpakBuffer,
                                   (unsigned char *)"This is a test", 14) == 0) {
                            putline("Ping received!");
                        }else{
                            putline("Unknown packet");
                        }

                        if (StopOnNextRx){
                            RadioStopReception(msg.SubMsg);
                            putline("returned in stop");
                            StopOnNextRx = FALSE;
                        }


                        break;

                    case MESSAGE_SENT:
                        // mpakTag = msg.SubMsg
                        RimSprintf(buffer, sizeof(buffer), "Tag %d sent", msg.SubMsg);
                        putline(buffer);
                        break;

                    case MESSAGE_NOT_SENT:
                        // mpakTag = msg.SubMsg
                        // reason = msg.Data[0]
                        RimSprintf(buffer, sizeof(buffer),"Tag %d failed: %d", msg.SubMsg, msg.Data[0]);
                        putline(buffer);
                        break;

                    case SIGNAL_LEVEL:
                        // RSSI = msg.SubMsg
                        RimSprintf(buffer, sizeof(buffer),"RSSI = %d", msg.SubMsg);
                        putline(buffer);
                        break;

                    case NETWORK_STARTED:
                        break;

                    case BASE_STATION_CHANGE:
                        putline("Base station changed");
                        break;

                    case RADIO_TURNED_OFF:
                        putline("Radio turned OFF");
                        break;
                }
                break;
        }

        // If any radio call returned a negative value, then this indicates an 
        // error occured in the API call.  Display the error.
        if (retval < 0){
            switch(retval){
                case RADIO_APP_NOT_REGISTERED:
                    putline("Error: Not registered");
                    break;

                case RADIO_MPAK_NOT_FOUND:
                    putline("Error: wrong/old Mpak tag?");
                    break;

                case RADIO_NO_FREE_BUFFERS:
                    putline("Error: No free buffers");
                    break;

                case RADIO_BAD_DATA:
                    putline("Error: Bad MPAK data");
                    break;

                case RADIO_BAD_TAG:
                    putline("Error: Illegal mpak tag");
                    break;

                case RADIO_ERROR_GENERAL:
                    putline("Error: General error");
                    break;

                case RADIO_ILLEGAL_SKIPNUM:
                    putline("Error: Illegal skipnum value");
                    break;
            }
        }
    }
}








//------------------------------------------------------------------
// Get info from the radio
//------------------------------------------------------------------
void PagerMain(void)
{
    // Create two instances of the radio task.  This shows 
    // how multiple apps connected to the radio interact.

    RimCreateThread(RadioTestTask,2000);

    RadioTestTask();
}
